import os
import warnings
from numbers import Number
from collections import OrderedDict
from os.path import join, dirname
import sys

import numpy as np
import pandas as pd

from Orange.data import TimeVariable, Table
from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Input, Output

from AnyQt.QtWidgets import QTreeWidget, \
    QWidget, QPushButton, QListView, QVBoxLayout
from AnyQt.QtGui import QIcon, QPalette
from AnyQt.QtCore import QSize, pyqtSignal, QTimer, Slot

from AnyQt.QtWidgets import QApplication, QStyle, QFileDialog, QLabel, QGridLayout, QTextBrowser
from PyQt5.QtCore import (
    Qt, QFileInfo, QTimer, QSettings, QObject, QSize, QMimeDatabase, QMimeType
)
import Orange.data

class OWBatchImport(widget.OWWidget):
    name = '批量导入'
    description = "数据批量导入"
    icon = 'icons/batchimport.svg'
    priority = 90

    # 定义输出信号
    class Outputs:
        data = Output("Data", Orange.data.Table, doc="Table", default=True)  #输出Orange Table格式的数据
        dataFrame = Output("Data Frame", pd.DataFrame, doc="DataFrame")          #输出Pandas DataFrame格式的数据

    # 不绘制内容区域
    want_main_area = False

    def __init__(self):
        grid = QGridLayout()
        self.dir_label = QLabel("目录:", self)
        self.browse_button = QPushButton(
            "…", icon=self.style().standardIcon(QStyle.SP_DirOpenIcon),
            toolTip="Browse filesystem", autoDefault=False,
        )
        self.browse_button.clicked.connect(self.browse)
        grid.addWidget(self.dir_label, 0, 1, 1, 1)
        grid.addWidget(self.browse_button, 0, 2, 1, 1)
        self.controlArea.layout().addLayout(grid)

        box = gui.widgetBox(self.controlArea, "Info", addSpace=False)
        self.summary_text = QTextBrowser(
            verticalScrollBarPolicy=Qt.ScrollBarAsNeeded,
            readOnly=True,
        )
        self.summary_text.viewport().setBackgroundRole(QPalette.NoRole)
        self.summary_text.setFrameStyle(QTextBrowser.NoFrame)
        self.summary_text.setMinimumHeight(self.fontMetrics().ascent() * 2 + 4)
        self.summary_text.viewport().setAutoFillBackground(False)
        box.layout().addWidget(self.summary_text)

    @Slot()
    def browse(self, prefixname=None, directory=None):
        """
        Open a file dialog and select a user specified file.
        """
        dir_path = QFileDialog.getExistingDirectory(self, "选择目录", "~/map")
        self.dir_label.setText(dir_path)

        # 将指定目录中的csv文件集合导入并转为pandas的dataframe对象
        df = self._get_data(dir_path)
        self.summary_text.setText("共载入{}条记录。".format(df.shape[0]))

        # 向数据通道发送数据
        self.Outputs.dataFrame.send(df)
        self.Outputs.data.send(pandas_to_table(df))

    def _get_data(self, dir_path):
        filecsv_list = []
        for root, dirs, files in os.walk(dir_path):
            for file in files:
                if os.path.splitext(file)[1] == '.csv':
                    filecsv_list.append(os.path.join(root, file))
        data = pd.DataFrame()
        for csv in filecsv_list:
            df_tmp = pd.read_csv(csv, header=0, encoding='utf-8')
            data = data.append(df_tmp, ignore_index=True)

        return data

from pandas.api import types as pdtypes
import Orange
def pandas_to_table(df):
    # type: (pd.DataFrame) -> Orange.data.Table
    """
    Convert a pandas.DataFrame to a Orange.data.Table instance.
    """
    index = df.index
    if not isinstance(index, pd.RangeIndex):
        df = df.reset_index()

    columns = []  # type: List[Tuple[Orange.data.Variable, np.ndarray]]

    for header, series in df.items():  # type: (Any, pd.Series)
        if pdtypes.is_categorical_dtype(series):
            coldata = series.values  # type: pd.Categorical
            categories = [str(c) for c in coldata.categories]
            var = Orange.data.DiscreteVariable.make(
                str(header), values=categories
            )
            # Remap the coldata into the var.values order/set
            coldata = pd.Categorical(
                coldata.astype("str"), categories=var.values
            )
            codes = coldata.codes
            assert np.issubdtype(codes.dtype, np.integer)
            orangecol = np.array(codes, dtype=np.float)
            orangecol[codes < 0] = np.nan
        elif pdtypes.is_datetime64_any_dtype(series):
            # Check that this converts tz local to UTC
            series = series.astype(np.dtype("M8[ns]"))
            coldata = series.values  # type: np.ndarray
            assert coldata.dtype == "M8[ns]"
            mask = np.isnat(coldata)
            orangecol = coldata.astype(np.int64) / 10 ** 9
            orangecol[mask] = np.nan
            var = Orange.data.TimeVariable.make(str(header))
            var.have_date = var.have_time = 1
        elif pdtypes.is_object_dtype(series):
            coldata = series.fillna('').values
            assert isinstance(coldata, np.ndarray)
            orangecol = coldata
            var = Orange.data.StringVariable.make(str(header))
        elif pdtypes.is_integer_dtype(series):
            coldata = series.values
            var = Orange.data.ContinuousVariable.make(str(header))
            var.number_of_decimals = 0
            orangecol = coldata.astype(np.float64)
        elif pdtypes.is_numeric_dtype(series):
            orangecol = series.values.astype(np.float64)
            var = Orange.data.ContinuousVariable.make(str(header))
        else:
            warnings.warn(
                "Column '{}' with dtype: {} skipped."
                    .format(header, series.dtype),
                UserWarning
            )
            continue
        columns.append((var, orangecol))

    cols_x = [(var, col) for var, col in columns if var.is_primitive()]
    cols_m = [(var, col) for var, col in columns if not var.is_primitive()]

    variables = [v for v, _ in cols_x]
    if cols_x:
        X = np.column_stack([a for _, a in cols_x])
    else:
        X = np.empty((df.shape[0], 0), dtype=np.float)
    metas = [v for v, _ in cols_m]
    if cols_m:
        M = np.column_stack([a for _, a in cols_m])
    else:
        M = None

    domain = Orange.data.Domain(variables, metas=metas)
    return Orange.data.Table.from_numpy(domain, X, None, M)

def main(argv=None):  # pragma: no cover
    app = QApplication(argv or [])
    w = OWBatchImport()
    w.show()
    w.raise_()
    app.exec_()
    w.saveSettings()
    w.onDeleteWidget()
    return 0


if __name__ == "__main__":  # pragma: no cover
    sys.exit(main(sys.argv))


from cProfile import label
from datetime import datetime, timedelta, date
import sys

from math import sin, cos, pi

import os
import tempfile
import numpy as np
import pandas as pd

from AnyQt.QtWidgets import (
    QGraphicsView, QGraphicsScene, QGraphicsSimpleTextItem,
    QGraphicsEllipseItem, QLabel, QGridLayout)
from AnyQt.QtGui import QColor, QPainter, QPixmap
from AnyQt.QtCore import Qt, QSize
from AnyQt.QtCore import QDate
from AnyQt.QtWidgets import QDateEdit, QComboBox, QFormLayout


import Orange.data
from Orange.statistics import contingency, distribution

from Orange.widgets import widget, gui
from Orange.widgets.settings import DomainContextHandler, ContextSetting, \
    Setting
from Orange.widgets.utils.itemmodels import DomainModel
from Orange.widgets.widget import Input, Output

import eventstudy as es
from Orange.data.io import FileFormat
from Orange.data.pandas_compat import table_from_frame, table_to_frame


# SCALE = 200


class OWAbnormalRet(widget.OWWidget):
    name = "异常收益AR"
    description = "自动下载股票交易数据，计算给定窗口期的超额收益。"
    keywords = ["pie chart", "chart", "visualisation"]
    icon = "icons/misc.svg"
    priority = 700

    class Inputs:
        data = Input("Stk & Mkt Data", Orange.data.Table)
        #ff = Input('FamaFrench', Orange.data.Table)
        #event = Input('EnentRecord', Orange.data.Table)
        
    class Outputs:
        es_result = Output("事件研究结果", Orange.data.Table)
        
        
    class Error(widget.OWWidget.Error):
        download_error = widget.Msg('数据下载错误.\n'
                                    '断网了？股票代码错误？')

    # settingsHandler = DomainContextHandler()
    # graph_name = "scene"
    
    is_price = Setting(True)
    log_return = Setting(True)
    window_start = Setting(-2)
    window_end = Setting(30)
    event_date = Setting('20200910')

    def __init__(self):
        super().__init__()
        self.dataset = None
        self.es_model = es.Single
        self.security_ticker = None
        self.market_ticker = None

        self.varlist = DomainModel(
            valid_types=Orange.data.Variable, separators=False)
        cb = gui.comboBox(
            self.controlArea, self, "security_ticker", box='公司股票代码：',
            model=self.varlist, callback=self.update_scene, contentsLength=12)        

        #cs = gui.comboBox(
        #    self.controlArea, self, "market_ticker", box="市场指数：",
        #    model=self.varlist, callback=self.update_scene)
        
        gui.lineEdit(
            self.controlArea, self, 'event_date', box = '事件时间：',
            tooltip='字符串表示的事件时间'
        )

        box = gui.vBox(self.controlArea, '事件窗口：')
        gui.spin(
            box, self, 'window_start',
            -100, 100, label='事件窗口开始:',
            controlWidth=60, alignment=Qt.AlignRight,
            callback=self.update_scene)
        
        gui.spin(
            box, self, 'window_end',
            0, 1000, label='事件窗口结束:',
            controlWidth=60, alignment=Qt.AlignRight,
            callback=self.update_scene)
        
        self.price_checkbox = gui.checkBox(
            self.controlArea, self, "is_price", "输入的为股票价格指数", box=True,
            callback=self.update_scene)
        
        self.logret_checkbox = gui.checkBox(
            self.controlArea, self, "log_return", "取对数收益", box=True,
            callback=self.update_scene)
        
        gui.rubber(self.controlArea)
        gui.widgetLabel(
            gui.hBox(self.controlArea, box=True),
            """为单只股票计算事件异常收益；
            也可以输入多只股票的事件日期，一次计算多只
            """)

        # self.scene = QGraphicsScene()
        # self.view = QGraphicsView(self.scene)
        # self.view.setRenderHints(
        #    QPainter.Antialiasing | QPainter.TextAntialiasing |
        #    QPainter.SmoothPixmapTransform)
        # self.mainArea.layout().addWidget(self.view)
        # self.mainArea.setMinimumWidth(400)
  
    # def sizeHint(self):
    #     return QSize(200, 150)  # Horizontal size is regulated by mainArea

    @Inputs.data
    def set_data(self, dataset):
        if dataset is not None and (
                not bool(dataset) or not len(dataset.domain.variables)):
            dataset = None
        self.closeContext()
        self.dataset = dataset
        self.is_prcice = None
        self.log_return = None
        domain = dataset.domain if dataset is not None else None
        self.varlist.set_domain(domain)
        #self.time_vars.set_domain(domain)
        if dataset is not None:
            self.select_default_variables(domain)
            self.openContext(self.dataset)
        with tempfile.NamedTemporaryFile(mode='w+', suffix='.csv',delete=True) as fp:
            ext = os.path.splitext(fp.name)[1]
            writer = FileFormat.writers.get(ext)
            writer.write(fp.name, self.dataset, False)
            self.es_model.import_returns(fp.name, is_price=self.is_price, log_return=self.log_return)  

    def select_default_variables(self, domain):
        if len(self.varlist) > len(domain.class_vars):
            first_attr = self.time_vars[len(domain.class_vars)]
        else:
            first_attr = None
        if len(self.varlist):
            self.security_ticker, self.market_ticker = self.varlist[0], first_attr
        else:
            self.security_ticker, self.market_ticker = self.market_ticker, None

    def update_scene(self):
        #self.scene.clear()
        if self.security_ticker is None:
            return
        self.es_model.constant_mean(
            security_ticker=self.security_ticker,
            event_date=self.event_date,
            event_window=(self.window_start,self.window_end),
            estimation_size=300,
        )
        result = table_from_frame(self.es_model.results())
        self.Outputs.es_result.send(result)


def main(argv=None):
    from AnyQt.QtWidgets import QApplication
    if argv is None:
        argv = sys.argv
    argv = list(argv)
    app = QApplication(argv)
    filename = "heart_disease"
    data = Orange.data.Table(filename)
    w = OWAbnormalRet()
    w.show()
    w.raise_()
    w.set_data(data)
    w.handleNewSignals()
    rval = app.exec_()
    w.set_data(None)
    w.handleNewSignals()
    w.saveSettings()
    return rval


if __name__ == "__main__":
    sys.exit(main())


from collections import OrderedDict

from AnyQt.QtWidgets import QComboBox, QFormLayout

from orangewidget.utils.widgetpreview import WidgetPreview

from Orange.widgets import widget, gui
from Orange.widgets.widget import Output

from orangecontrib.timeseries import Timeseries
from Orange.data.pandas_compat import table_from_frame
import akshare as ak


class OWMacroUSA(widget.OWWidget):
    name = '美国宏观数据'
    description = "从 Akshare下载美国经济统计数据"
    icon = 'icons/美国宏观.svg'
    priority = 9

    class Outputs:
        time_series = Output("Time series", Timeseries)


    Macro_DICT = OrderedDict((
        ('美国GDP',     ak.macro_usa_gdp_monthly),
        ('美国CPI',   ak.macro_usa_cpi_monthly),
        ('美国核心CPI',      ak.macro_china_fdi),
        ('美国个人支出',      ak.macro_usa_personal_spending),
        ('美国零售销售',      ak.macro_usa_retail_sales),
        ('美国进口物价指数',      ak.macro_usa_import_price),
        ('美国出口价格指数',      ak.macro_usa_export_price),
        ('LMCI',      ak.macro_usa_lmci),
        ('美国失业率',      ak.macro_usa_unemployment_rate),
        ('美国挑战者企业裁员人数',      ak.macro_usa_job_cuts),
        ('美国非农就业人数',      ak.macro_usa_non_farm),
        ('美国ADP就业人数',      ak.macro_usa_adp_employment),
        ('美国核心PCE物价指数',      ak.macro_usa_core_pce_price),
        ('美国实际个人消费支出',      ak.macro_usa_real_consumer_spending),
        ('美国贸易帐',      ak.macro_usa_trade_balance),
        ('美国经常帐',      ak.macro_usa_current_account),
        ('贝克休斯钻井报告',      ak.macro_usa_rig_count),
        ('美国PPI',      ak.macro_usa_ppi),
        ('美国核心PPI',      ak.macro_usa_core_ppi),
        ('美国API原油库存',      ak.macro_usa_api_crude_stock),
        ('美国Markit制造业PMI初值',      ak.macro_usa_pmi),
        ('美国ISM制造业PMI',      ak.macro_usa_ism_pmi),
        ('美国工业产出',      ak.macro_usa_industrial_production),
        ('美国耐用品订单',      ak.macro_usa_durable_goods_orders),
        ('美国工厂订单',      ak.macro_usa_factory_orders),
        ('美国Markit服务业PMI',      ak.macro_usa_services_pmi),
        ('美国商业库存',      ak.macro_usa_business_inventories),
        ('美国ISM非制造业PMI',      ak.macro_usa_ism_non_pmi),
        ('美国NAHB房产市场指数',      ak.macro_usa_nahb_house_market_index),
        ('美国新屋开工总数',      ak.macro_usa_house_starts),
        ('美国新屋销售总数',      ak.macro_usa_new_home_sales),
        ('美国营建许可总数',      ak.macro_usa_building_permits),
        ('美国成屋销售总数',      ak.macro_usa_exist_home_sales),
        ('美国FHFA房价指数',      ak.macro_usa_house_price_index),
        ('美国S&P/CS20座大城市房价指数',      ak.macro_usa_spcs20),
        ('美国成屋签约销售指数',      ak.macro_usa_pending_home_sales),
        ('未决房屋销售',      ak.macro_usa_phs),
        ('美国谘商会消费者信心指数',      ak.macro_usa_cb_consumer_confidence),
        ('美国NFIB小型企业信心指数',      ak.macro_usa_nfib_small_business),
        ('美国密歇根大学消费者信心指数',      ak.macro_usa_michigan_consumer_sentiment),
        ('美国EIA原油库存',      ak.macro_usa_eia_crude_rate),
        ('美国初请失业金人数',      ak.macro_usa_initial_jobless),
        ('美国原油产量',      ak.macro_usa_crude_inner),
    ))


    want_main_area = False
    resizing_enabled = False

    class Error(widget.OWWidget.Error):
        download_error = widget.Msg('数据下载错误.\n'
                                    '断网了？代码错误？')

    def __init__(self):
        layout = QFormLayout()
        gui.widgetBox(self.controlArea, True, orientation=layout)

        self.MacroIdx = MacroIdx =QComboBox()
        MacroIdx.addItems(self.Macro_DICT.keys())
        MacroIdx.setCurrentIndex(0)
        layout.addRow("宏观经济指标:", self.MacroIdx)


        self.button = gui.button(
            self.controlArea, self, '下载数据', callback=self.download)

    def download(self):
        # 
        MacroIdx = self.MacroIdx.currentText()
        self.Error.clear()

        with self.progressBar(3) as progress:
            try:
                progress.advance()
                self.button.setDisabled(True)
                df = self.Macro_DICT[MacroIdx]()
                data = Timeseries.from_data_table(table_from_frame(df))
                data.name = MacroIdx
                self.Outputs.time_series.send(data)
            except Exception as e:
                self.Error.download_error()
            finally:
                self.button.setDisabled(False)


if __name__ == "__main__":
    WidgetPreview(OWMacroUSA).run()


from os.path import join, dirname
import os
import sys
import tempfile
from plotly.io import to_html
import plotly.graph_objs as go

from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import OWWidget, Input, Output

from AnyQt.QtWidgets import QTreeWidget, \
    QWidget, QPushButton, QListView, QVBoxLayout
from AnyQt.QtGui import QIcon

from Orange.widgets.utils.itemmodels import VariableListModel
#from orangecontrib.finecon.widgets.utls.echarts import Echarts
from Orange.data import Table, Domain, DiscreteVariable, Variable, \
    ContinuousVariable
    
from orangecontrib.timeseries import Timeseries
from Orange.data.pandas_compat import table_from_frame
import akshare as ak

class OWLineChartEcharts(OWWidget):
    name = '异常收益AR'
    description = "下载股票交易数据，计算异常收益AR."
    icon = 'icons/misc.svg'
    priority = 90

    class Inputs:
        data = Input("Data", Table)
    class Outputs:
        time_series = Output("Time series", Timeseries)

    attrs = settings.Setting({})  # Maps data.name -> [attrs]

    # 设置输入数据:股票代码，事件日，自然日还是股票交易日，前排除时长，后排除时长，时间前期，时间后期
    attr_x = ''
    attr_y = ''

    def __init__(self):
        self.data = None
        self.tmp_file = tempfile.NamedTemporaryFile(mode='w', suffix='.html',delete=False)
        # A model for displaying python list like objects in Qt item view classes
        # 用于表示下拉框数据列表的模型
        self.varmodel = VariableListModel(parent=self)
        
        vbox = gui.vBox(self)
        # 从输入数据字段列表选取x轴的下拉框
        self.cb_attr_x = gui.comboBox(vbox, self, 'attr_x',
                     label='x轴:',
                     orientation='horizontal',
                     model=self.varmodel,
                     sendSelectedValue=True)
        # 从输入数据字段列表选取y轴的下拉框
        self.cb_attr_y = gui.comboBox(vbox, self, 'attr_y',
                     label='y轴:',
                     orientation='horizontal',
                     model=self.varmodel,
                     sendSelectedValue=True)
        # 按钮
        self.draw_button = QPushButton('绘制折线图', self)
        self.draw_button.clicked.connect(self.linechart_plot)
        # 在控制区域放置控制部件
        self.controlArea.layout().addWidget(vbox)
        self.controlArea.layout().addWidget(self.draw_button)
        # 创建Echarts部件
        self.chart = None #Echarts(self)
        # 在显示区域放置Echarts部件
        self.mainArea.layout().addWidget(self.chart)

    def set_figure(self, fig=None):
        self.tmp_file.seek(0)
        if fig is None:
            fig=go.Figure()
        fig.update_xaxes()
        
    def linechart_plot(self):
        # Echarts的参数选项
        options = {
            'xAxis': {
                'data': self.data.get_column_view(self.attr_x)[0].tolist(),
            },
            'yAxis': {
                'scale': 'true'
            },
            'dataZoom': [
                {
                    'type': 'inside'
                },
            ],
            'series': [{
                'symbolSize': 10,
                'data': self.data.get_column_view(self.attr_y)[0].tolist(),
                'type': 'line'
            }]
        };
        # 绘制图形
        self.chart.chart(options=options)

    @Inputs.data
    def set_data(self, data):
        new_data = None if data is None else data
        if new_data is not None and self.data is not None \
                and new_data.domain == self.data.domain:
            self.data = new_data
            for config in self.configs:
                config.selection_changed()
            return

        self.data = data = None if data is None else data
        if data is None:
            self.varmodel.clear()
            self.chart.clear()
            return

        self.varmodel.wrap([var for var in data.domain.variables
                            if var.is_continuous])

if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication
    from orangecontrib.timeseries import ARIMA, VAR

    a = QApplication([])
    ow = OWLineChartEcharts()

    table = Table("iris")
    ow.set_data(table)

    ow.show()
    a.exec()


import unittest
from Orange.widgets.tests.base import WidgetTest

from orangecontrib.finecon.widgets.pyjulia import OWPyJuliaScript


class ExampleTests(unittest.TestCase):
    def test_addition(self):
        self.assertEqual(1 + 1, 2)


class TestMyWidget(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWPyJuliaScript)

    def test_addition(self):
        self.assertEqual(1 + 1, 2)



from json import dumps as json
from collections import defaultdict
from collections.abc import MutableMapping
from os.path import join, dirname

import numpy as np

from AnyQt.QtCore import QObject, pyqtSlot
from AnyQt.QtWidgets import QApplication
from Orange.widgets.utils.webview import WebviewWidget

def _Autotree():
    return defaultdict(_Autotree)


def _merge_dicts(master, update):
    """Merge dicts recursively in place (``master`` is modified)"""
    for k, v in master.items():
        if k in update:
            if isinstance(v, MutableMapping) and isinstance(update[k], MutableMapping):
                update[k] = _merge_dicts(v, update[k])
    master.update(update)
    return master


def _kwargs_options(kwargs):
    """Transforma a dict into a hierarchical dict.

    Example
    -------
    >>> (_kwargs_options(dict(a_b_c=1, a_d_e=2, x=3)) ==
    ...  dict(a=dict(b=dict(c=1), d=dict(e=2)), x=3))
    True
    """
    kwoptions = _Autotree()
    for kws, val in kwargs.items():
        cur = kwoptions
        kws = kws.split('_')
        for kw in kws[:-1]:
            cur = cur[kw]
        cur[kws[-1]] = val
    return kwoptions

class Echarts(WebviewWidget):
    """Create a Echarts webview widget.

    Parameters
    ----------
    parent: QObject
        Qt parent object, if any.
    bridge: QObject
        Exposed as ``window.pybridge`` in JavaScript.
    options: dict
        Default options for this chart. If any option's value is a string
        that starts with an empty block comment ('/**/'), the expression
        following is evaluated in JS.
    javascript: str
        Additional JavaScript code to evaluate beforehand. If you
        need something exposed in the global namespace,
        assign it as an attribute to the ``window`` object.
    debug: bool
        Enables right-click context menu and inspector tools.
    **kwargs:
        The additional options. The underscores in argument names imply
        hierarchy, e.g., keyword argument such as ``chart_type='area'``
        results in the following object, in JavaScript::

            {
                chart: {
                    type: 'area'
                }
            }

        The original `options` argument is updated with options from
        these kwargs-derived objects.
    """

    _ECHARTS_HTML = join(dirname(__file__), '_echarts', 'chart.html')

    def __init__(self,
                 parent=None,
                 bridge=None,
                 # options=None,
                 javascript='',
                 debug=False,
                 **kwargs):
        super().__init__(parent, bridge, debug=debug)
        # options = (options or {}).copy()
        # 载入网页模板，创建echarts对象
        with open(self._ECHARTS_HTML) as html:
            self.setHtml(html.read() % dict(javascript=javascript),
                         self.toFileURL(dirname(self._ECHARTS_HTML)) + '/')

    def _update_options_dict(self, options, enable_zoom, enable_select,
                             enable_point_select, enable_rect_select,
                             enable_scrollbar, kwargs):
        if enable_zoom:
            _merge_dicts(options, _kwargs_options(dict(
                mapNavigation_enableMouseWheelZoom=True,
                mapNavigation_enableButtons=False)))
        if enable_select:
            _merge_dicts(options, _kwargs_options(dict(
                chart_events_click='/**/unselectAllPoints/**/')))
        if enable_point_select:
            _merge_dicts(options, _kwargs_options(dict(
                plotOptions_series_allowPointSelect=True,
                plotOptions_series_point_events_click='/**/clickedPointSelect/**/')))
        if enable_rect_select:
            _merge_dicts(options, _kwargs_options(dict(
                chart_zoomType=enable_rect_select,
                chart_events_selection='/**/rectSelectPoints/**/')))
        if kwargs:
            _merge_dicts(options, _kwargs_options(kwargs))

        if not enable_scrollbar:
            _merge_dicts(options, {'scrollbar':
                                       {'enabled': enable_scrollbar}})

    def exposeObject(self, name, obj):
        if isinstance(obj, np.ndarray):
            # chokes on NaN values. Instead it prefers 'null' for
            # points it is not intended to show.
            obj = obj.astype(object)
            obj[np.isnan(obj)] = None
        super().exposeObject(name, obj)

    def chart(self, data=None, options=None, **kwargs):
        """ 载入数据绘制图形.
        """
        options = (options or {}).copy()
        if not isinstance(options, MutableMapping):
            raise ValueError('options must be dict')
        if kwargs:
            _merge_dicts(options, _kwargs_options(kwargs))
        # 将传入的python对象装为js对象并载入网页的上下文
        self.exposeObject('pydata', data)
        self.exposeObject('pyoption', options)
        # 执行js语句绘制图形
        self.evalJS('''
            var data = pydata;
            var option = pyoption;
            myChart.setOption(option);
        ''')

def main():
    """ A simple test. """
    app = QApplication([])

    def _on_selected_points(points):
        print(len(points), points)

    xAxis = {
        'type': 'category',
        'data': ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
    }
    yAxis = {
        'type': 'value'
    }
    series = [{
        'data': [820, 932, 901, 934, 1290, 1330, 1320],
        'type': 'line'
    }]
    options = {
        'xAxis': xAxis,
        'yAxis': yAxis,
        'series': series
    }
    w = Echarts(debug=True)
    w.chart(options=options)
    w.show()
    app.exec()


if __name__ == '__main__':
    main()



from datetime import datetime, timedelta, date
from collections import OrderedDict

from AnyQt.QtCore import QDate
from AnyQt.QtWidgets import QDateEdit, QComboBox, QFormLayout

from orangewidget.utils.widgetpreview import WidgetPreview

from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output

from orangecontrib.timeseries import Timeseries
from Orange.data.pandas_compat import table_from_frame
import akshare as ak


class OWakshare(widget.OWWidget):
    name = 'A股交易数据'
    description = "从Akshare--东方财富 下载经济统计和金融交易数据"
    icon = 'icons/A股交易数据.svg'
    priority = 9

    class Outputs:
        time_series = Output("Time series", Timeseries)

    QT_DATE_FORMAT = 'yyyy-MM-dd'
    PY_DATE_FORMAT = '%Y-%m-%d'  
    AK_DATE_FROMAT = '%Y%m%d'   #akshare只接受 20200909 这样的字符串日期。
    MIN_DATE = date(1851, 1, 1)

    date_from = settings.Setting(
        (datetime.now().date() - timedelta(5 * 365)).strftime(PY_DATE_FORMAT))
    date_to = settings.Setting(datetime.now().date().strftime(PY_DATE_FORMAT))
    symbols = settings.Setting(['600857', '000088'])

    Adjust_Dict = {
        '不复权':'', 
        '前复权':'qfq', 
        '后复权':'hfq'
    }

    ADJUST_DICT = OrderedDict((
        ('不复权', ''),
        ('前复权','qfq'),
    ))

    want_main_area = False
    resizing_enabled = False

    class Error(widget.OWWidget.Error):
        download_error = widget.Msg('数据下载错误.\n'
                                    '断网了？股票代码错误？')

    def __init__(self):
        layout = QFormLayout()
        gui.widgetBox(self.controlArea, True, orientation=layout)

        self.combo = combo = QComboBox(
            editable=True, insertPolicy=QComboBox.InsertAtTop)
        combo.addItems(self.symbols)
        layout.addRow("股票代码:", self.combo)

        self.QPeriod = QPeriod =QComboBox()
        QPeriod.addItems(['daily', 'weekly', 'monthly'])
        QPeriod.setCurrentIndex(0)
        layout.addRow("数据频率:", self.QPeriod)

        minDate = QDate.fromString(self.MIN_DATE.strftime(self.PY_DATE_FORMAT),
                                   self.QT_DATE_FORMAT)
        date_from, date_to = (
            QDateEdit(QDate.fromString(date, self.QT_DATE_FORMAT),
                      displayFormat=self.QT_DATE_FORMAT, minimumDate=minDate,
                      calendarPopup=True)
            for date in (self.date_from, self.date_to))

        @date_from.dateChanged.connect
        def set_date_from(date):
            self.date_from = date.toString(self.QT_DATE_FORMAT)

        @date_to.dateChanged.connect
        def set_date_to(date):
            self.date_to = date.toString(self.QT_DATE_FORMAT)

        layout.addRow("起始日期:", date_from)
        layout.addRow("结束日期:", date_to)

        self.Adjust = Adjust = QComboBox()
        Adjust.addItems(tuple(self.Adjust_Dict.keys()))
        Adjust.setCurrentIndex(0)
        layout.addRow("复权方式:", Adjust)

        self.button = gui.button(
            self.controlArea, self, '下载数据', callback=self.download)

    def download(self):
        date_from = datetime.strptime(self.date_from, self.PY_DATE_FORMAT)
        date_from = date_from.strftime(self.AK_DATE_FROMAT)
        date_to = datetime.strptime(self.date_to, self.PY_DATE_FORMAT)
        date_to = date_to.strftime(self.AK_DATE_FROMAT)

        #  股票代码数据处理
        symbol = self.combo.currentText().strip().upper()
        self.combo.removeItem(self.combo.currentIndex())
        self.combo.insertItem(0, symbol)
        self.combo.setCurrentIndex(0)

        # 时间频率和复权数据
        period = self.QPeriod.currentText().strip().lower()
        adjust = self.Adjust_Dict[self.Adjust.currentText()]
        try:
            self.symbols.remove(symbol)
        except ValueError:
            pass
        self.symbols.insert(0, symbol)

        if not symbol:
            return

        self.Error.clear()
        with self.progressBar(3) as progress:
            try:
                progress.advance()
                self.button.setDisabled(True)
                df = ak.stock_zh_a_hist(symbol=symbol, period=period, start_date=date_from, end_date=date_to, adjust=adjust)
                data = Timeseries.from_data_table(table_from_frame(df))
                data.name = symbol
                self.Outputs.time_series.send(data)
            except Exception as e:
                self.Error.download_error()
            finally:
                self.button.setDisabled(False)


if __name__ == "__main__":
    WidgetPreview(OWakshare).run()


from datetime import datetime, timedelta, date
from collections import OrderedDict

from AnyQt.QtCore import QDate
from AnyQt.QtWidgets import QDateEdit, QComboBox, QFormLayout

from orangewidget.utils.widgetpreview import WidgetPreview

from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output

from orangecontrib.timeseries import Timeseries
from Orange.data.pandas_compat import table_from_frame
import akshare as ak


class OWakshare(widget.OWWidget):
    name = '财务报表及指标'
    description = "从Akshare--下载A股财务报表和财务指标数据。"
    icon = 'icons/财务数据.svg'
    priority = 9

    class Outputs:
        time_series = Output("Time series", Timeseries)


    symbols = settings.Setting(['600857', '000088'])

    Fin_DICT = OrderedDict((
        ('资产负债表-新浪', lambda stk:ak.stock_financial_report_sina(stock=stk, symbol="资产负债表")),
        ('利润表-新浪', lambda stk:ak.stock_financial_report_sina(stock=stk, symbol="利润表")),
        ('现金流量表-新浪', lambda stk:ak.stock_financial_report_sina(stock=stk, symbol="现金流量表")),
        ('资产负债表-东财', lambda stk:ak.stock_balance_sheet_by_report_em(symbol=('SH'+stk if  stk.startswith('6') else 'SZ'+stk))),
        ('利润表-东财', lambda stk:ak.stock_profit_sheet_by_report_em(symbol=('SH'+stk if  stk.startswith('6') else 'SZ'+stk))),
        ('利润表-年-东财', lambda stk:ak.stock_profit_sheet_by_yearly_em(symbol=('SH'+stk if  stk.startswith('6') else 'SZ'+stk))),
        ('现金流量表-东财', lambda stk:ak.stock_cash_flow_sheet_by_report_em(symbol=('SH'+stk if  stk.startswith('6') else 'SZ'+stk))),
        ('财务摘要-新浪', lambda stk:ak.stock_financial_abstract(stock=stk)),
        ('财务指标-新浪', lambda stk:ak.stock_financial_analysis_indicator(symbol=stk)),
    ))

    want_main_area = False
    resizing_enabled = False

    class Error(widget.OWWidget.Error):
        download_error = widget.Msg('数据下载错误.\n'
                                    '断网了？股票代码错误？')

    def __init__(self):
        layout = QFormLayout()
        gui.widgetBox(self.controlArea, True, orientation=layout)

        self.combo = combo = QComboBox(
            editable=True, insertPolicy=QComboBox.InsertAtTop)
        combo.addItems(self.symbols)
        layout.addRow("股票代码:", self.combo)

        self.Idx = Idx =QComboBox()
        Idx.addItems(self.Fin_DICT.keys())
        Idx.setCurrentIndex(0)
        layout.addRow("类型:", self.Idx)

        self.button = gui.button(
            self.controlArea, self, '下载数据', callback=self.download)

    def download(self):
        #  股票代码数据处理
        symbol = self.combo.currentText().strip().upper()
        self.combo.removeItem(self.combo.currentIndex())
        self.combo.insertItem(0, symbol)
        self.combo.setCurrentIndex(0)

        # 时间频率和复权数据
        idx_fun = self.Fin_DICT[self.Idx.currentText()]
        try:
            self.symbols.remove(symbol)
        except ValueError:
            pass
        self.symbols.insert(0, symbol)

        if not symbol:
            return

        self.Error.clear()
        with self.progressBar(3) as progress:
            try:
                progress.advance()
                self.button.setDisabled(True)
                df = idx_fun(symbol)
                data = Timeseries.from_data_table(table_from_frame(df))
                data.name = symbol
                self.Outputs.time_series.send(data)
            except Exception as e:
                self.Error.download_error()
            finally:
                self.button.setDisabled(False)


if __name__ == "__main__":
    WidgetPreview(OWakshare).run()


#!/usr/bin/env python

from os import path, walk

import sys
from setuptools import setup, find_packages

try:
    # need recommonmark for build_htmlhelp command
    import recommonmark
except ImportError:
    pass



NAME = "Orange3-Finecon"

VERSION = "0.5.9"
# 0.3.x 增加调用R的方式，但是图形貌似会有一些问题？
## 0.3.7 修正R脚本中含有时间的对象转换失败。


AUTHOR = 'He Ray'
AUTHOR_EMAIL = 'hcmray@qq.com'

URL = 'http://orange.biolab.si/download'
DESCRIPTION = "Add-on containing 金融及经济 widgets"
LONG_DESCRIPTION = open(path.join(path.dirname(__file__), 'README.pypi'),
                        'r', encoding='utf-8').read()

LICENSE = "MIT"

KEYWORDS = (
    # [PyPi](https://pypi.python.org) packages with keyword "orange3 add-on"
    # can be installed using the Orange Add-on Manager
    'FinEcon',
    '金融计量分析',
    '金融数据分析'
)

PACKAGES = find_packages()

PACKAGE_DATA = {
    'orangecontrib.finecon': ['tutorials/*.ows'],
    'orangecontrib.finecon.widgets': ['icons/*'],
}

DATA_FILES = [
    # Data files that will be installed outside site-packages folder
]

INSTALL_REQUIRES = sorted(set(
    line.partition('#')[0].strip()
    for line in open(path.join(path.dirname(__file__), 'requirements.txt'))
) - {''})

ENTRY_POINTS = {
    # Entry points that marks this package as an orange add-on. If set, addon will
    # be shown in the add-ons manager even if not published on PyPi.
    'orange3.addon': (
        'FinEcon = orangecontrib.finecon',
    ),
    # Entry point used to specify packages containing tutorials accessible
    # from welcome screen. Tutorials are saved Orange Workflows (.ows files).
    'orange.widgets.tutorials': (
        # Syntax: any_text = path.to.package.containing.tutorials
        'exampletutorials = orangecontrib.finecon.tutorials',
    ),

    # Entry point used to specify packages containing widgets.
    'orange.widgets': (
        # Syntax: category name = path.to.package.containing.widgets
        # Widget category specification can be seen in
        #    orangecontrib/FinEcon/widgets/__init__.py
        'FinEcon = orangecontrib.finecon.widgets',
    ),

    # Register widget help
    "orange.canvas.help": (
        'html-index = orangecontrib.finecon.widgets:WIDGET_HELP_PATH',)
}

NAMESPACE_PACKAGES = ["orangecontrib"]

TEST_SUITE = "orangecontrib.finecon.tests.suite"


def include_documentation(local_dir, install_dir):
    global DATA_FILES

    doc_files = []
    for dirpath, _, files in walk(local_dir):
        doc_files.append(
            (
                dirpath.replace(local_dir, install_dir),
                [path.join(dirpath, f) for f in files],
            )
        )
    DATA_FILES.extend(doc_files)


if __name__ == '__main__':
    include_documentation('doc/_build/html', 'help/orange3-FinEcon')
    setup(
        name=NAME,
        version=VERSION,
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        url=URL,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type='text/markdown',
        license=LICENSE,
        packages=PACKAGES,
        package_data=PACKAGE_DATA,
        data_files=DATA_FILES,
        install_requires=INSTALL_REQUIRES,
        entry_points=ENTRY_POINTS,
        keywords=KEYWORDS,
        namespace_packages=NAMESPACE_PACKAGES,
        zip_safe=False,
    )



from collections import OrderedDict

from AnyQt.QtWidgets import QComboBox, QFormLayout

from orangewidget.utils.widgetpreview import WidgetPreview

from Orange.widgets import widget, gui
from Orange.widgets.widget import Output

from orangecontrib.timeseries import Timeseries
from Orange.data.pandas_compat import table_from_frame
import akshare as ak


class OWMacroCN(widget.OWWidget):
    name = '中国宏观数据'
    description = "从 Akshare--东方财富 下载经济统计数据"
    icon = 'icons/中国宏观.svg'
    priority = 9

    class Outputs:
        time_series = Output("Time series", Timeseries)


    Macro_DICT = OrderedDict((
        ('中国宏观杠杆率',     ak.macro_cnbs),
        ('企业商品价格指数',   ak.macro_china_qyspjg),
        ('外商直接投资',      ak.macro_china_fdi),
        ('社会融资规模增量',      ak.macro_china_shrzgm),
        ('LPR',      ak.macro_china_lpr),
        ('GDP—年',      ak.macro_china_gdp_yearly),
        ('CPI—月',      ak.macro_china_cpi_monthly),
        ('PPI—年',      ak.macro_china_ppi_yearly),
        ('出口—美元—年',      ak.macro_china_exports_yoy),
        ('进口—美元—年',      ak.macro_china_imports_yoy),
        ('贸易帐—美元—年',      ak.macro_china_trade_balance),
        ('工业增加值',      ak.macro_china_gyzjz),
        ('规模以上工业增加值',      ak.macro_china_industrial_production_yoy),
        ('官方制造业PMI',      ak.macro_china_pmi_yearly),
        ('财新制造业PMI终值',      ak.macro_china_cx_pmi_yearly),
        ('财新服务业PMI',      ak.macro_china_cx_services_pmi_yearly),
        ('官方非制造业PMI',      ak.macro_china_non_man_pmi),
        ('外汇储备-亿美元',      ak.macro_china_fx_reserves_yearly),
        ('M2',      ak.macro_china_m2_yearly),
        ('新房价指数',      ak.macro_china_new_house_price),
        ('企业景气及企业家信心指数',      ak.macro_china_enterprise_boom_index),
        ('全国税收收入',      ak.macro_china_national_tax_receipts),
        ('银行理财产品发行数量',      ak.macro_china_bank_financing),
        ('原保险保费收入',      ak.macro_china_insurance_income),
        ('手机出货量',      ak.macro_china_mobile_number),
        ('菜篮子产品批发价格指数',      ak.macro_china_vegetable_basket),
        ('农产品批发价格总指数',      ak.macro_china_agricultural_product),
        ('农副指数',      ak.macro_china_agricultural_index),
        ('能源指数',      ak.macro_china_energy_index),
        ('大宗商品价格',      ak.macro_china_commodity_price_index),
        ('费城半导体指数',      ak.macro_global_sox_index),
        ('义乌小商品指数-电子元器件',      ak.macro_china_yw_electronic_index),
        ('建材指数',      ak.macro_china_construction_index),
        ('建材价格指数',      ak.macro_china_construction_price_index),
        ('物流景气指数',      ak.macro_china_lpi_index),
        ('原油运输指数',      ak.macro_china_bdti_index),
        ('超灵便型船运价指数',      ak.macro_china_bsi_index),
        ('新增信贷',      ak.macro_china_new_financial_credit),
        ('CPI',      ak.macro_china_cpi),
        ('GDP',      ak.macro_china_gdp),
        ('PPI',      ak.macro_china_ppi),
        ('采购经理人指数',      ak.macro_china_pmi),
        ('城镇固定资产投资',      ak.macro_china_gdzctz),
        ('海关进出口增减情况',      ak.macro_china_hgjck),
        ('财政收入',      ak.macro_china_czsr),
        ('外汇贷款',      ak.macro_china_whxd),
        ('本外币存款',      ak.macro_china_wbck),
        ('货币净投放与净回笼',      ak.macro_china_hb),
        ('央行公开市场操作',      ak.macro_china_gksccz),
        ('新债发行',      ak.macro_china_bond_public),
        ('消费者信心指数',      ak.macro_china_xfzxx),
        ('存款准备金率',      ak.macro_china_reserve_requirement_ratio),
        ('社会消费品零售总额',      ak.macro_china_consumer_goods_retail),
        ('全社会用电分类情况表',      ak.macro_china_society_electricity ),
        ('全社会客货运输量',      ak.macro_china_society_traffic_volume ),
        ('邮电业务基本情况',      ak.macro_china_postal_telecommunicational ),
        ('国际旅游外汇收入构成',      ak.macro_china_international_tourism_fx ),
        ('民航客座率及载运率',      ak.macro_china_passenger_load_factor ),
        ('航贸运价指数',      ak.macro_china_freight_index ),
        ('央行货币当局资产负债',      ak.macro_china_central_bank_balance ),
        ('保险业经营情况',      ak.macro_china_insurance ),
        ('货币供应量',      ak.macro_china_supply_of_money ),
        ('FR007利率互换曲线',      ak.macro_china_swap_rate ),
        ('央行黄金和外汇储备',      ak.macro_china_foreign_exchange_gold ),
        ('商品零售价格指数',      ak.macro_china_retail_price_index ),
        ('国房景气指数',      ak.macro_china_real_estate ),
        ('外汇和黄金储备',      ak.macro_china_fx_gold ),
        ('中国货币供应量',      ak.macro_china_money_supply ),
        ('全国股票交易统计表',      ak.macro_china_stock_market_cap ),
        ('SHIBOR',      ak.macro_china_shibor_all ),
        ('人民币香港银行同业拆息',      ak.macro_china_hk_market_info ),
        ('人民币汇率中间价',      ak.macro_china_rmb),
        ('深圳融资融券报告',      ak.macro_china_market_margin_sz),
        ('上海融资融券报告',      ak.macro_china_market_margin_sh),
        ('上海黄金交易所报告',      ak.macro_china_au_report),
        ('全国综合电煤价格指数',      ak.macro_china_ctci),
        ('各价区电煤价格指数',      ak.macro_china_ctci_detail),
        ('历史电煤价格指数',      ak.macro_china_ctci_detail_hist),
    ))


    want_main_area = False
    resizing_enabled = False

    class Error(widget.OWWidget.Error):
        download_error = widget.Msg('数据下载错误.\n'
                                    '断网了？代码错误？')

    def __init__(self):
        layout = QFormLayout()
        gui.widgetBox(self.controlArea, True, orientation=layout)

        self.MacroIdx = MacroIdx =QComboBox()
        MacroIdx.addItems(self.Macro_DICT.keys())
        MacroIdx.setCurrentIndex(0)
        layout.addRow("宏观经济指标:", self.MacroIdx)


        self.button = gui.button(
            self.controlArea, self, '下载数据', callback=self.download)

    def download(self):
        # 
        MacroIdx = self.MacroIdx.currentText()
        self.Error.clear()

        with self.progressBar(3) as progress:
            try:
                progress.advance()
                self.button.setDisabled(True)
                df = self.Macro_DICT[MacroIdx]()
                data = Timeseries.from_data_table(table_from_frame(df))
                data.name = MacroIdx
                self.Outputs.time_series.send(data)
            except Exception as e:
                self.Error.download_error()
            finally:
                self.button.setDisabled(False)


if __name__ == "__main__":
    WidgetPreview(OWMacroCN).run()